#-*- coding: utf-8 -*-
#
# perceptron_sonar_7.py

from os import chdir
from os import listdir

# Se déplacer dans le dossier en cours si besoin
path = r'P:\PRO\USB\NSI\NSI 1ERE\100300 NSI IA PROJET\NSI_ML_PREPA_DEPART_ELEVES_ML_4'
chdir(path)

# Perceptron Algorithm on the Sonar Dataset
from random import seed
from random import randrange
from csv import reader

# Load a CSV file
def load_csv(filename):
	dataset = list()
	with open(filename, 'r') as file:
		csv_reader = reader(file)
		for row in csv_reader:
			if not row:
				continue
			dataset.append(row)
	return dataset

# Convert string column to float
def str_column_to_float(dataset):
	'''
	version PG
	'''
	for row in dataset:
		for colonne in range(len(dataset[0])-1):
			row[colonne]=float(row[colonne])
	
	
# Convert string column to integer
def str_column_to_int(dataset):
	'''
	version PG
	'''
	for row in dataset:
		if row[-1]=='M':
			row[-1]=1
		else:
			row[-1]=0


def impression_fold(dataset,lignes=None):
	'''
	Impression formatée de quelques lignes du dataset :
	
	Impression de quelques éléments du dataset
	------------------------------------------
						0        1        2                 58       59       60   
	Row numéro   1 :  0.04530  0.05230  0.08430    ....    0.00520  0.00440  1.00000     M
	Row numéro   2 :  0.02620  0.05820  0.10990    ....    0.00950  0.00780  1.00000     M
	Row numéro   3 :  0.01000  0.01710  0.06230    ....    0.00400  0.01170  1.00000     M
	Row numéro   4 :  0.07620  0.06660  0.04810    ....    0.01070  0.00940  1.00000     M
	Row numéro 205 :  0.05220  0.04370  0.01800    ....    0.00770  0.00310  0.00000     R
	Row numéro 206 :  0.03030  0.03530  0.04900    ....    0.00360  0.00480  0.00000     R
	
	'''
	colonne_debut=(0,1,2)
	colonne_fin=(58,59,60)
	
	# Si la liste lignes est vide alors on affiche toutes
	# les lignes du dataset donc on créé la liste lignes
	# en y mettant par une compréhension de liste tous les
	# numéros des lignes
	if lignes==None:
		lignes=[i for i in range(len(dataset))]
	
		
	print()
	print("Impression de quelques éléments du dataset")
	print("------------------------------------------")
	
	print(" "*17,end='')
	for _ in colonne_debut:
		print("   %3d   " % (_), end='')
	print(" "*10,end='')
	for _ in colonne_fin:
		print("   %3d   " % (_), end='')	
	print()
	for numero_de_lignes in lignes:
		print("Row numéro %3d : " % (numero_de_lignes),end='')
		for colonne in colonne_debut:
			print("%8.5f " % (dataset[numero_de_lignes][colonne]),end='')
		print("   ....   ",end="")
		for colonne in colonne_fin:
			print("%8.5f " % (dataset[numero_de_lignes][colonne]),end='')
		if dataset[numero_de_lignes][-1]==1:
			print('    M')
		else:
			print('    R')

			


# Split a dataset into k folds
def cross_validation_split(datas, n_folds):
	'''
	Répartition du jeux d'échantillons dataset_copy en k folds
	dataset_copy n'est pas modifié. La fonction retourne une liste
	de folds contenant chacun fold_size échantillons de mesures.
	'''
	dataset_split = []

	'''
	Recopier votre code
	'''
	
	# On retourne les k-folds datasets
	return dataset_split




# Make a prediction with weights
def predict(row, weights):	
	activation = weights[0]
	for i in range(len(row)-1):
		activation += weights[i + 1] * row[i]
	return 1.0 if activation >= 0.0 else 0.0
	
# Estimate Perceptron weights using stochastic gradient descent
def train_weights_historique(train, l_rate, n_epoch, weights):
	for _ in range(n_epoch):
		for row in train:
			prediction = predict(row, weights)
			error = row[-1] - prediction
			weights[0] = weights[0] + l_rate * error
			for i in range(len(row)-1):
				weights[i + 1] = weights[i + 1] + l_rate * error * row[i]
	return weights

# Calculate accuracy percentage
def accuracy_metric(actual, predicted):
	'''
	A compléter
	'''
	return ..........

## Test the Perceptron algorithm on the sonar dataset

valeur_seed = 1
seed(valeur_seed)

#=========================================
# Saisie et pré-traitements des données
#=========================================

# Chargement et préparation des données
filename = 'sonar.all-data.csv'
dataset = load_csv(filename)

# Conversion des mesures en float
str_column_to_float(dataset)
	
# Conversion des classifications en integer
str_column_to_int(dataset)

# Recopie du dataset dans dataset_copy pour préserver la liste initiale
dataset_copy = dataset.copy()

# Répartition en k groupes
n_folds = 8
folds = cross_validation_split(dataset_copy, n_folds)
nombre_de_folds = len(folds)
nombre_de_data_par_fold = len(folds[0])

# Réglage de la descente de gradient du perceptron
l_rate = 0.1
n_epoch = 100

# Impression des données de l'expérimentation
print()
print("-"*90)
print("| Machine Learning | Modèle : Perceptron | Analyse : k-folds | Data : %15s |" % (filename))
print("-"*90)
print("P.G Avril 2021")
print()
print("Nombre de folds : ",n_folds)
print("Réglage seed : ",valeur_seed)
print("l_rate : ",l_rate)
print("Epochs : ",n_epoch)
print()

#=========================================
# Initialisation
#=========================================

# A) Initialisation des poids
weights_initial = [0.0 for i in range(len(folds[0][0]))]

# B) Initialisation de la liste des n° de folds
liste_depart = [x for x in range(n_folds)]

#=========================================
# Balayage de tous les numéros de fold, 
# chacun sert de test à tour de rôle
#=========================================

for index in range(len(liste_depart)):
	
	# A) Répartition n° fold entre test : fold_test 
	#                et entraînement : fold_entrainement
	liste_copy = liste_depart.copy()
	fold_test = index
	liste_copy.pop(index)
	fold_entrainement = liste_copy
	print("Itération : %2d n° fold test : %2d" % (index,fold_test),\
	" n° folds entrainement : ", fold_entrainement," | ", end="")

	# B) Apprentissage sur les folds d'entraînement
	for no_fold in fold_entrainement:
		data_train = folds[no_fold]
		weights_calcule = train_weights_historique(data_train, l_rate, n_epoch, weights_initial )
		weights_initial  = weights_calcule
	
	# C) Test du modèle avec le fold de test
	test = folds[fold_test] 
	actual = []
	predicted = []
	# Pour toutes les données du fold de test 
	for row in test:
		# Calcul de la prédiction
		prediction = predict(row, weights_calcule)
		predicted.append(prediction) 	
		reel = row[-1]
		actual.append(reel)
	
	# D) Calcul de la précision
	precision = accuracy_metric(actual, predicted)
	print("Précision de la prédiction : %8.2f%%" % (precision))

#=========================================
# Evaluation finale sur le dataset complet
#=========================================

# A) prédiction
test = dataset
actual = []
predicted = []

# Pour toutes les données du fold de test 
for row in test:
	prediction = predict(row, weights_calcule)
	predicted.append(prediction) 	
	reel = row[-1]
	actual.append(reel)

# B) Calcul précision
precision = accuracy_metric(actual, predicted)

# C) Sortie résultats
print()	
print("Précision de la prédiction sur le dataset complet : %8.2f%%" % (precision))
print()	
print()
print("Remerciements à M. Jason Brownlee")
print("Machine Learning Algorithms From Scratch With Python")
print("https://machinelearningmastery.com/machine-learning-algorithms-from-scratch/")
print()